/******************************************************************************/
/*  NetSvrEdit                                                                */
/*  This program will provide the user with the ability to change netserver   */
/*  configurations, and also provide the current information.                 */
/******************************************************************************/

#include <iostream.h>                //Include for cout, cin functions.
#include <iomanip.h>                 //Included for setw() function.
#include <cwb.h>                     //Include for client access constants
#include <cwbco.h>                   //Include for system list functions
#include <cwbrc.h>                   //Remote command API's
#include <cwbcosys.h>                //System object API's
#include <cwbsv.h>                   //Error handle API's
#include <cwbsy.h>                   //Max system name, etc.
#include <cwbdt.h>                   //Data conversion for program parameters
#include <cwbnlcnv.h>                //Conversion of EBCDIC string data
#include <string.h>                  //String manipulation functions


//Define maximum sizes for various parameters
const MAX_SYSTEMS = 20;
const MAX_MESSAGE = 200;
const MAX_USER = 10;
const MAX_COMMAND = 200;
const MAX_PATHNAME = 200;
const MAX_SHARE = 12;
const MAX_SERVER = 15;
const MAX_DOMAIN = 15;
const MAX_DESCRIPTION = 50;
const MAX_IP = 15;

//Function prototypes
void printMenu();                    //Print options
void changeName();                   //Change Netserver name
void changeGuest();                  //Change Guest profile
void executeCommand(char *szCommand);//Used by menu options to run Netserver APIs.
void startServer();                  //Start Netserver
void endServer();                    //End Netserver
void addShare();                     //Add File Share
char getDigit(int i);                //Convert a value from 0-15 to corresponding Hex digit
void intToHex(int i);                //Convert an integer value to a string representation in hex
void getInformation();               //Retrieve server information
unsigned long bytesToInt(unsigned char* byteArray);  //Convert raw bytes from server to int value
int connectServer();                 //Connect to AS/400
void listShares();                   //Get share information

//Global variables
char *szSystem = "";                 //System name to be used throughout.
char szBuffer[9] = "";               //Global variable where results of intToHex are stored....should be changed.
char szInput[25] = "";               //Char array to hold various user inputs
char **nullchar = '\0';              //Constant for newline character.
cwbCO_SysHandle sysHandle = 0;       //Global system handle

int main()
{
    unsigned int rc = 0;             //Variable to use for all API return codes
	int iSystem = 0;                 //Variable to hold selected system index
	int iSelection = 0;              //Variable to hold selected menu option

	cout << "This program will allow you to modify netserver" << endl
		 << "settings and file shares." << endl << endl
		 << "Choose a system:" << endl << endl;

	//Retrieve list of available systems
    cwbCO_SysListHandle ListHandle = 0;                         //Declare Syslist handle
	rc = cwbCO_CreateSysListHandle(&ListHandle, 0);             //Create it.
	if(rc != CWB_OK)                                            //Verify that handle was created
	{
		cout << "Error creating system list!" << endl;
		return -1;
	}

	unsigned long size = 0;                                     //Var to hold system list size
	rc = cwbCO_GetSysListSize(ListHandle, &size);               //Determine list size
	if(rc != CWB_OK)                                            //Check for API errors
	{
		cout << "Error retrieving list size!" << endl;
		return -1;
	}

	char szSysList[MAX_SYSTEMS][CWBCO_MAX_SYS_NAME + 1];        //Create an array to hold system names.
                                                                //MAXSYSTEMS*MAXNAMELENGTH

	unsigned long lNeeded = 0;                                  //Output buffer for GetNextSysName
	unsigned long j = 0;                                        //Loop counter

	//As long as there are valid system names, continue retrieving up to max # of systems
	while((j < size) && (rc != CWBCO_END_OF_LIST) && (j <= MAX_SYSTEMS))
	{
		rc = cwbCO_GetNextSysNameA(ListHandle, szSysList[j], strlen(szSysList[j]), &lNeeded);
		j++;
	}
    
    //Print out a list of systems.
	unsigned long lNamesRetrieved = j;
	for(j = 0; j < lNamesRetrieved; j++)
	{
		cout << "\t" << j + 1 << ".  " << szSysList[j] << endl;
	}

	cout << endl << "\t99. Quit" << endl;
	cout << endl;

	//Request input
	cout << "Selection: ";
	cin.getline(szInput, 25);

	//Convert string input to integer value
    iSystem = strtol(szInput, nullchar, 10);

	//Check for requested end of program
	if(iSystem == 99)
	{
		cout << "Program ended." << endl;
		return 0;
	}

	//Loop until a valid menu option is selected
    while((iSystem <= 0)||(iSystem > (int)lNamesRetrieved))
	{
		cout << "Invalid selection, select again." << endl << "Selection: ";

		cin.getline(szInput, 25);
		iSystem = strtol(szInput, nullchar, 10);

		if(iSystem == 99)
		{	
			cout << "Program ended." << endl;
			return 0;
		}
	}

    //Assign global variable the selected system name
	szSystem = szSysList[iSystem - 1];

	//Show selection and give menu options
	cout << "Selected system -> " << szSystem << endl << endl;
    printMenu();

	//Request input
	cin.getline(szInput, 25);
	iSelection = strtol(szInput, nullchar, 10);

	//Connect to selected server
	int iConnect = 0;
	if(iSelection != 99)
	{
		iConnect = connectServer();
	}
	
	if(iConnect != 0)
	{
		return -1;
	}

	//As long a user does not ask to quit, continue to print menu and request input
	while(iSelection != 99)
	{
		//Check selection and call appropriate function
		switch(iSelection)
		{
		case 1:
			startServer();
			break;
		case 2:
			endServer();
			break;
		case 3:
			changeName();
			break;
		case 4:
			changeGuest();
			break;
		case 5:
			addShare();
			break;
		case 6:
			getInformation();
			break;
		case 7:
			listShares();
			break;
		default:
			cout << "Invalid selection!" << endl << endl;
		}

		printMenu();

        //Get next selection
		cin.getline(szInput, 25);
		iSelection = strtol(szInput, nullchar, 10);
	}

	//Delete system object
	rc = cwbCO_DeleteSystem(sysHandle);
	if(rc != CWB_OK)
	{
		cout << "Error releasing system object!" << endl;
	}	

	cout << "Program ended." << endl;
	return 0;
}

/****************************************************************/
/*  Function printMenu()                                        */
/*  Simple function to provide list of available options        */
/****************************************************************/
void printMenu()
{
	cout << endl
		 << "Select an option:" << endl << endl
		 << "\t1.  Start Netserver." << endl
		 << "\t2.  End Netserver."  << endl
		 << "\t3.  Change Netserver name information." << endl
		 << "\t4.  Change guest profile.             " << endl
		 << "\t5.  Add file share." << endl 
		 << "\t6.  List server information." << endl 
		 << "\t7.  List file share information." << endl << endl
		 << "\t99. Quit" << endl << endl
		 << "Selection: ";
}

/****************************************************************/
/*  Function int connectServer()                                */
/*  Creates system object and signs on to system.               */
/****************************************************************/
int connectServer()
{
	unsigned int rc = 0;
	unsigned long errCount = 0;
	cwbSV_ErrHandle errHandle = 0;

	//Create a system object
	rc = cwbCO_CreateSystem(szSystem, &sysHandle);
    if(rc != CWB_OK)
	{
		cout << "Error creating system object!" << endl;
		return -1;
	}

	//Create an error handle
	rc = cwbSV_CreateErrHandle(&errHandle);
    if(rc != CWB_OK)
	{
		cout << "Error creating error handle!" << endl;
		return -1;
	}

	//Signon to system
	rc = cwbCO_Signon(sysHandle, errHandle);
	if(rc != CWB_OK)
	{
		cout << "Error signing on!" << endl;
		return -1;
	}

	//Check for any errors
	rc = cwbSV_GetErrCount(errHandle, &errCount);
    if(rc == CWB_OK)
	{
		if(errCount >= 1)
		{
			char szMessage[MAX_MESSAGE + 1] = "";
			rc = cwbSV_GetErrText(errHandle, szMessage, MAX_MESSAGE + 1, NULL);
			if(rc == CWB_OK)
			{
				cout << szMessage << endl;
			}
		}
	}

	return 0;
}


/****************************************************************/
/*  Function executeCommand(char *szCommand)                    */
/*  Executes the command in the buffer pointed to by szCommand  */
/*  Will create a system object, connect, and execute.          */
/****************************************************************/
void executeCommand(char *szCommand)
{
	unsigned int rc = 0;
	unsigned long errCount = 0;
	cwbSV_ErrHandle errHandle = 0;
	cwbRC_SysHandle rcSysHandle = 0;

	//Create an error handle
	rc = cwbSV_CreateErrHandle(&errHandle);
    if(rc != CWB_OK)
	{
		cout << "Error creating error handle!" << endl;
		return;
	}

	//Start a remote command conversation
	rc = cwbRC_StartSysEx(sysHandle, &rcSysHandle);
	if(rc != CWB_OK)
	{
		cout << "Error starting Remote Command job!" << endl;
		return;
	}

	//Execute the command
	rc = cwbRC_RunCmd(rcSysHandle, szCommand, errHandle);
	if(rc != CWB_OK)
	{
		cout << "Error executing command!"  << endl;
	}
	else
	{
		cout << "Command executed successfully." << endl;
	}

	//Check for any errors
	rc = cwbSV_GetErrCount(errHandle, &errCount);
    if(rc == CWB_OK)
	{
		if(errCount >= 1)
		{
			char szMessage[MAX_MESSAGE + 1] = "";
			rc = cwbSV_GetErrText(errHandle, szMessage, MAX_MESSAGE + 1, NULL);
			if(rc == CWB_OK)
			{
				cout << szMessage << endl << endl;
			}
		}
	}

	//Shut down the remote command job.
	rc = cwbRC_StopSys(rcSysHandle);
    if(rc != CWB_OK)
	{
		cout << "Error ending remote command job!" << endl;
	}

	//Get rid of error handle
	rc = cwbSV_DeleteErrHandle(errHandle);
	if(rc != CWB_OK)
	{
		cout << "Error releasing error handle!" << endl;
	}

}

/****************************************************************/
/*  Function startServer()                                      */
/*  Builds the command string to start netserver and then calls */
/*  function to execute.                                        */
/****************************************************************/
void startServer()
{
	char *szCommand = "CALL QZLSSTRS PARM('0' x'00000000')";
	executeCommand(szCommand);
}

/****************************************************************/
/*  Function endServer()                                        */
/*  Builds the command string to end netserver and calls the    */
/*  function to execute.                                        */
/****************************************************************/
void endServer()
{
    char *szCommand = "CALL QZLSENDS PARM(x'00000000')";
	executeCommand(szCommand);
}

/****************************************************************/
/*  Function changeGuest()                                      */
/*  Prompts for information needed to change guest profile,     */
/*  builds the command string, and calls function to execute    */
/*  the command.                                                */
/****************************************************************/
void changeGuest()
{
	char szUser[MAX_USER + 1] = "";
    char szCommand[MAX_COMMAND + 1] = "CALL QZLSCHSG PARM(";

    cout << "New guest profile: ";	
	cin.getline(szUser, MAX_USER);

	strcat(szCommand, szUser);
	char *szAppend = " x'00000000')";
	strcat(szCommand, szAppend);

	executeCommand(szCommand);
}

/****************************************************************/
/*  Function addShare()                                         */
/*  Prompts for information needed to add a file share, builds  */
/*  the command string, and then calls function to execute the  */
/*  command.                                                    */
/****************************************************************/
void addShare()
{
	char szCommand[MAX_COMMAND + 1] = "CALL QZLSADFS PARM(";
	char szShareName[MAX_SHARE + 1] = "";
	char szPathName[MAX_PATHNAME + 1] = "";
	char szDescription[MAX_PATHNAME + 1] = "";
    char szHexOutput[9] = "        ";
    int iRead = 0;

	cout << "Share Name: ";
	cin.getline(szShareName, MAX_SHARE);

	cout << "Path: ";
	cin.getline(szPathName, MAX_PATHNAME);

	cout << "Description: ";
	cin.getline(szDescription, MAX_PATHNAME);

	while((iRead != 2) && (iRead != 1))
	{
		cout << "Read (1) or Read/Write (2): ";
		cin.getline(szInput, 25);
		iRead = strtol(szInput, nullchar, 10);
	}

	strcat(szCommand, szShareName);
	char *szAppend = " '";
	strcat(szCommand, szAppend);
	strcat(szCommand, szPathName);
	szAppend = "' x'";
	strcat(szCommand, szAppend);

	int length = strlen(szPathName);
	intToHex(length);
    strcat(szCommand, szBuffer);

	szAppend = "' x'00000000' '";
	strcat(szCommand, szAppend);
	strcat(szCommand, szDescription);
	
	if(iRead==1)
	{
		szAppend = "' x'00000001' x'FFFFFFFF' x'00000000')";
	}
	else
	{
		szAppend = "' x'00000002' x'FFFFFFFF' x'00000000')";
	}

	strcat(szCommand, szAppend);

	executeCommand(szCommand);
}

/****************************************************************/
/*  Function changeName()                                       */
/*  Prompts for information needed to change the Netserver      */
/*  name, builds the command string, and then calls function    */
/*  to execute the command.                                     */
/****************************************************************/
void changeName()
{
    char szCommand[MAX_COMMAND + 1] = "CALL QZLSCHSN PARM(";
	char szServerName[MAX_SERVER + 1];
	char szDomain[MAX_DOMAIN + 1];
	char szDescription[MAX_DESCRIPTION + 1];

	cout << "Server Name: ";
	cin.getline(szServerName, MAX_SERVER);

	cout << "Domain Name: ";
	cin.getline(szDomain, MAX_DOMAIN);

	cout << "Description: ";
	cin.getline(szDescription, MAX_DESCRIPTION);

	//Build the command
	strcat(szCommand, szServerName);
	char *szAppend = " ";
	strcat(szCommand, szAppend);
	strcat(szCommand, szDomain);
	szAppend = " '";
	strcat(szCommand, szAppend);
	strcat(szCommand, szDescription);
	szAppend = "' x'00000000')";
	strcat(szCommand, szAppend);

	executeCommand(szCommand);

}

/****************************************************************/
/*  Function getInformation()                                   */
/*  Establishes a connection, creates a user space, dumps       */
/*  Netserver information to the user space, retrieves, and     */
/*  cleans up.                                                  */
/****************************************************************/
void getInformation()
{
	unsigned int rc = 0;
	unsigned long errCount = 0;
	cwbSV_ErrHandle errHandle = 0;
	cwbRC_SysHandle rcSysHandle = 0;
    cwbRC_PgmHandle pgmHandle = 0;

	//Create an error handle
	rc = cwbSV_CreateErrHandle(&errHandle);
    if(rc != CWB_OK)
	{
		cout << "Error creating error handle!" << endl;
		return;
	}

	//Start a remote command conversation
	rc = cwbRC_StartSysEx(sysHandle, &rcSysHandle);
	if(rc != CWB_OK)
	{
		cout << "Error starting Remote Command job!" << endl;
		return;
	}

	//Create program to create a user space
	char szProgram[11] = "QUSCRTUS";
	char szLibrary[11] = "QSYS";
	rc = cwbRC_CreatePgm(szProgram, szLibrary, &pgmHandle);
	if(rc != CWB_OK)
	{
		cout << "Error creating program!" << endl;
		return;
	}

	//Add the parameters
	//Parm 1 - User space to create
	unsigned char szName[20] = {0xD5, 0xC5, 0xE3, 0xE2, 0xE5, 0xD9, 0xC5, 0xC4, 0xC9, 0xE3,   //"NETSVREDIT"
		                        0xD8, 0xE3, 0xC5, 0xD4, 0xD7, 0x40, 0x40, 0x40, 0x40, 0x40};  //"QTEMP     "
    
	rc = cwbRC_AddParm(pgmHandle, CWBRC_INPUT, 20, szName);
	if(rc != CWB_OK)
	{
		cout << "Error on QUSCRTUS parameter!" << endl;
		return;
	}

	//Parm 2 - Extended attributes
    unsigned char szAttrib[10] = {0x40, 0x40, 0x40, 0x40, 0x40, 0x40, 0x40, 0x40, 0x40, 0x40}; //"          "
	rc = cwbRC_AddParm(pgmHandle, CWBRC_INPUT, 10, szAttrib);
	if(rc != CWB_OK)
	{
		cout << "Error on QUSCRTUS parameter!" << endl;
		return;
	}

	//Parm 3 - Initial size
	unsigned long lSpaceSize = 0xD0070000;       //Make it 2000 bytes long
	rc = cwbRC_AddParm(pgmHandle, CWBRC_INPUT, 4, (unsigned char*)&lSpaceSize);
	if(rc != CWB_OK)
	{
		cout << "Error on QUSCRTUS parameter!" << endl;
		return;
	}

	//Parm 4 - Initial value of all bytes
	unsigned char cInitial = 0x40;
	rc = cwbRC_AddParm(pgmHandle, CWBRC_INPUT, 1, &cInitial);
	if(rc != CWB_OK)
	{
		cout << "Error on QUSCRTUS parameter!" << endl;
		return;
	}

	//Parm 5 - Initial public authority - *CHANGE
	unsigned char szAuthority[10] = {0x5C, 0xC3, 0xC8, 0xC1, 0xD5, 0xC7, 0xC5, 0x40, 0x40, 0x40};
	rc = cwbRC_AddParm(pgmHandle, CWBRC_INPUT, 10, szAuthority);
	if(rc != CWB_OK)
	{
		cout << "Error on QUSCRTUS parameter!" << endl;
		return;
	}

	//Parm 6 - Description - All blanks
	unsigned char szSpaceDesc[50] = {0x40, 0x40, 0x40, 0x40, 0x40, 0x40, 0x40, 0x40, 0x40, 0x40,
		                             0x40, 0x40, 0x40, 0x40, 0x40, 0x40, 0x40, 0x40, 0x40, 0x40,
								     0x40, 0x40, 0x40, 0x40, 0x40, 0x40, 0x40, 0x40, 0x40, 0x40,
								     0x40, 0x40, 0x40, 0x40, 0x40, 0x40, 0x40, 0x40, 0x40, 0x40,
								     0x40, 0x40, 0x40, 0x40, 0x40, 0x40, 0x40, 0x40, 0x40, 0x40};

	rc = cwbRC_AddParm(pgmHandle, CWBRC_INPUT, 50, szSpaceDesc);
	if(rc != CWB_OK)
	{
		cout << "Error on QUSCRTUS parameter!" << endl;
		return;
	}

	//Execute the program
	rc = cwbRC_CallPgm(rcSysHandle, pgmHandle, errHandle);
    if(rc != CWB_OK)
	{
		cout << "Error creating user space!" << endl;
		
		//Check for any errors
		rc = cwbSV_GetErrCount(errHandle, &errCount);
		if(rc == CWB_OK)
		{
			if(errCount >= 1)
			{
				char szMessage[MAX_MESSAGE + 1] = "";
				rc = cwbSV_GetErrText(errHandle, szMessage, MAX_MESSAGE + 1, NULL);
				if(rc == CWB_OK)
				{
					cout << szMessage << endl << endl;
				}
			}
		}
	}

	//Delete the program
	if ( cwbRC_DeletePgm(pgmHandle) != CWB_OK )
	{
		cout << "Error on cleanup!" << endl;
	}
 
	//Call program to dump Netserver information to user space
	char szCommand[MAX_COMMAND] = "CALL QZLSLSTI PARM('NETSVREDITQTEMP     ' ZLSL0200 *ALL x'00000000')";

	rc = cwbRC_RunCmd(rcSysHandle, szCommand, errHandle);
	if(rc != CWB_OK)
	{
		cout << "Error retrieving information!"  << endl;

		//Check for any errors
		rc = cwbSV_GetErrCount(errHandle, &errCount);
		if(rc == CWB_OK)
		{
			if(errCount >= 1)
			{
				char szMessage[MAX_MESSAGE + 1] = "";
				rc = cwbSV_GetErrText(errHandle, szMessage, MAX_MESSAGE + 1, NULL);
				if(rc == CWB_OK)
				{
					cout << szMessage << endl << endl;
				}
			}
		}
	}

    //Create a program to retrieve the user space information.
	char szProgram2[11] = "QUSRTVUS";
	char szLibrary2[11] = "QSYS";
	rc = cwbRC_CreatePgm(szProgram2, szLibrary2, &pgmHandle);
	if(rc != CWB_OK)
	{
		cout << "Error creating program!" << endl;
		return;
	}

    //Parm 1 - User space name, using szName declared previously
	rc = cwbRC_AddParm(pgmHandle, CWBRC_INPUT, 20, szName);
	if(rc != CWB_OK)
	{
		cout << "Error on QUSRTVUS parameter!" << endl;
		return;
	}

	//Parm 2 - Starting position in user space, first time get offset to data
	unsigned long lStart = 0x7D000000;
	rc = cwbRC_AddParm(pgmHandle, CWBRC_INPUT, 4, (unsigned char*)&lStart);
	if(rc != CWB_OK)
	{
		cout << "Error on QUSRTVUS parameter!" << endl;
		return;
	}

	//Parm 3 - Number of bytes to retrieve
	unsigned long lLength = 0x04000000;
	rc = cwbRC_AddParm(pgmHandle, CWBRC_INPUT, 4, (unsigned char*)&lLength);
	if(rc != CWB_OK)
	{
		cout << "Error on QUSRTVUS parameter!" << endl;
		return;
	}

	//Parm 4 - Variable to receive information
	unsigned char szOffset[4] = "";
	rc = cwbRC_AddParm(pgmHandle, CWBRC_OUTPUT, 4, szOffset);
	if(rc != CWB_OK)
	{
		cout << "Error on QUSRTVUS parameter!" << endl;
		return;
	}

	//Execute the program
	rc = cwbRC_CallPgm(rcSysHandle, pgmHandle, errHandle);
    if(rc != CWB_OK)
	{
		cout << "Error retrieving user space!" << endl;
		
		//Check for any errors
		rc = cwbSV_GetErrCount(errHandle, &errCount);
		if(rc == CWB_OK)
		{
			if(errCount >= 1)
			{
				char szMessage[MAX_MESSAGE + 1] = "";
				rc = cwbSV_GetErrText(errHandle, szMessage, MAX_MESSAGE + 1, NULL);
				if(rc == CWB_OK)
				{
					cout << szMessage << endl << endl;
				}
			}
		}
	}

	//Determine the offset value
	unsigned long lOffset = bytesToInt(&szOffset[0]);
	//Add 1 to the offset since user space index starts at 1
	lOffset = lOffset + 1;

    //Convert to AS400 parameter
	unsigned long lOffsetParm = 0;
	rc = cwbDT_Bin4ToBin4((char*)(void*)&lOffsetParm, (char*)(void*)&lOffset);
	if(rc != CWB_OK)
	{
		cout << "Error converting parameter!" << endl;
		return;
	}

	//Change parameters to retrieve at offset.
	rc = cwbRC_SetParm(pgmHandle, 1, CWBRC_INPUT, 4, (unsigned char*)&lOffsetParm);
	if(rc != CWB_OK)
	{
		cout << "Error assigning parameter!" << endl;
		return;
	}

	unsigned long lDataSize = 0xE0020000;
	rc = cwbRC_SetParm(pgmHandle, 2, CWBRC_INPUT, 4, (unsigned char*)&lDataSize);
	if(rc != CWB_OK)
	{
		cout << "Error assigning parameter!" << endl;
		return;
	}

	unsigned char szInformation[736] = "";
	rc = cwbRC_SetParm(pgmHandle, 3, CWBRC_OUTPUT, 736, szInformation);
	if(rc != CWB_OK)
	{
		cout << "Error assigning parameter!" << endl;
		return;
	}

	//Execute the program to retrieve Netserver info
	rc = cwbRC_CallPgm(rcSysHandle, pgmHandle, errHandle);
    if(rc != CWB_OK)
	{
		cout << "Error retrieving user space!" << endl;
		
		//Check for any errors
		rc = cwbSV_GetErrCount(errHandle, &errCount);
		if(rc == CWB_OK)
		{
			if(errCount >= 1)
			{
				char szMessage[MAX_MESSAGE + 1] = "";
				rc = cwbSV_GetErrText(errHandle, szMessage, MAX_MESSAGE + 1, NULL);
				if(rc == CWB_OK)
				{
					cout << szMessage << endl << endl;
				}
			}
		}
	}

	//Retrieve information from raw data in szInformation
	//Determine code page of process
	unsigned long lPCCodePage = 0;
	rc = cwbNL_GetCodePage(&lPCCodePage, 0);
	if(rc != CWB_OK)
	{
		cout << "Unable to retrieve pc code page!" << endl;
	}

	//Determine code page of AS400
	unsigned long lCCSID = 0;
	rc = cwbNL_GetHostCCSID(szSystem, &lCCSID);
	if(rc != CWB_OK)
	{
		cout << "Unable to retrieve host CCSID!" << endl;
	}

	unsigned long lHostCodePage = 0;
	rc = cwbNL_CCSIDToCodePage(lCCSID, &lHostCodePage, 0);
	if(rc != CWB_OK)
	{
		cout << "Unable to retrieve host code page!" << endl;
	}

    //Create a converter
	cwbNL_Converter cvtr = 0;
	char pad = ' ';
	rc = cwbNL_CreateConverter(lHostCodePage, lPCCodePage, &cvtr, 0, 0, 0, &pad);
	if(rc != CWB_OK)
	{
		cout << "Unable to create a converter!" << endl;
		return;
	}

	cout << endl << "\t\t\tCurrent\t\tPending" << endl;
	unsigned long lNeeded = 0;
	unsigned long lNumberErrors = 0;
	unsigned long lErrorIndex = 0;

    //Convert the data
	unsigned long lCodePage;
	unsigned long lNewCodePage;
	unsigned long lTimeout;
	unsigned long lNewTimeout;
	unsigned long lLockTimeout;
	unsigned long lNewLockTimeout;
	unsigned long lBrowsing;
	unsigned long lNewBrowsing;
	unsigned long lWINS;
	unsigned long lNewWINS;
	char szNetserverName[MAX_SERVER + 1] = "";
	char szNewName[MAX_SERVER + 1] = "";
    char szDomain[MAX_DOMAIN + 1] = "";
	char szNewDomain[MAX_DOMAIN + 1] = "";
	char szGuestName[MAX_USER + 1] = "";
	char szNewGuest[MAX_USER + 1] = "";
	char szWinsIP[MAX_IP + 1] = "";
	char szNewWinsIP[MAX_IP + 1] = "";
	char szWinsIP2[MAX_IP + 1] = "";
	char szNewWinsIP2[MAX_IP + 1] = ""
	;

	lCodePage = bytesToInt(&szInformation[0]);
	lNewCodePage = bytesToInt(&szInformation[4]);
	lTimeout = bytesToInt(&szInformation[8]);
	lNewTimeout = bytesToInt(&szInformation[12]);
	lLockTimeout = bytesToInt(&szInformation[16]);
	lNewLockTimeout = bytesToInt(&szInformation[20]);
	lBrowsing = bytesToInt(&szInformation[24]);
	lNewBrowsing = bytesToInt(&szInformation[28]);
	lWINS = bytesToInt(&szInformation[32]);
	lNewWINS = bytesToInt(&szInformation[36]);
	cwbNL_Convert(cvtr, 15, 16, (char*)&szInformation[228], szWinsIP, &lNumberErrors, &lErrorIndex, &lNeeded, 0);
	cwbNL_Convert(cvtr, 15, 16, (char*)&szInformation[243], szNewWinsIP, &lNumberErrors, &lErrorIndex, &lNeeded, 0);
	cwbNL_Convert(cvtr, 15, 16, (char*)&szInformation[258], szWinsIP2, &lNumberErrors, &lErrorIndex, &lNeeded, 0);
	cwbNL_Convert(cvtr, 15, 16, (char*)&szInformation[273], szNewWinsIP2, &lNumberErrors, &lErrorIndex, &lNeeded, 0);
	cwbNL_Convert(cvtr, 15, 16, (char*)&szInformation[68], szNetserverName, &lNumberErrors, &lErrorIndex, &lNeeded, 0);
	cwbNL_Convert(cvtr, 15, 16, (char*)&szInformation[83], szNewName, &lNumberErrors, &lErrorIndex, &lNeeded, 0);
    cwbNL_Convert(cvtr, 15, 16, (char*)&szInformation[98], szDomain, &lNumberErrors, &lErrorIndex, &lNeeded, 0);
	cwbNL_Convert(cvtr, 15, 16, (char*)&szInformation[113], szNewDomain, &lNumberErrors, &lErrorIndex, &lNeeded, 0);
    cwbNL_Convert(cvtr, 10, 11, (char*)&szInformation[48], szGuestName, &lNumberErrors, &lErrorIndex, &lNeeded, 0);
	cwbNL_Convert(cvtr, 10, 11, (char*)&szInformation[58], szNewGuest, &lNumberErrors, &lErrorIndex, &lNeeded, 0);

	cout << setiosflags( ios::left );
	cout << "Server Name:      \t" << setw(15) << szNetserverName << "\t" << szNewName << endl;
    cout << "Domain Name:      \t" << setw(15) << szDomain << "\t" << szNewDomain << endl;
	cout << "GuestProfile:     \t" << setw(15) << szGuestName << "\t" << szNewGuest << endl;
	cout << "Code Page:        \t" << setw(15) << lCodePage << "\t" << lNewCodePage << endl;
	cout << "Idle Time-out:    \t" << setw(15) << lTimeout << "\t" << lNewTimeout << endl;
	cout << "Lock time-out:    \t" << setw(15) << lLockTimeout << "\t" << lNewLockTimeout << endl;
	cout << "Browsing interval:\t" << setw(15) << lBrowsing << "\t" << lNewBrowsing << endl;
	
	if(lWINS==1)
	{
		cout << "WINS Enabled:   \t" << setw(15) << "YES" << "\t";
	}
	else
	{
		cout << "WINS Enabled:   \t" << setw(15) << "NO" << "\t";
	}
    if(lNewWINS==1)
	{
		cout << "YES" << endl;
	}
	else
	{
		cout << "NO" << endl;
	}

	cout << "Primary WINS:     \t" << setw(15) << szWinsIP << "\t" << szNewWinsIP << endl;
	cout << "Secondary WINS:    \t" << setw(15) << szWinsIP2 << "\t" << szNewWinsIP2 << endl;
	cout << endl;
	cout << "Enter to continue:";

	char szEnter[2];
	cin.getline(szEnter, 2);

	//Cleanup

	//Delete the converter
	rc = cwbNL_DeleteConverter(cvtr, 0);
	if(rc != CWB_OK)
	{
		cout << "Error deleting converter!" << endl;
	}

    //Delete the user space.
	char szCommand2[MAX_COMMAND] = "DLTUSRSPC QTEMP/NETSVREDIT";

	rc = cwbRC_RunCmd(rcSysHandle, szCommand2, errHandle);
	if(rc != CWB_OK)
	{
		cout << "Error on cleanup!"  << endl;

		//Check for any errors
		rc = cwbSV_GetErrCount(errHandle, &errCount);
		if(rc == CWB_OK)
		{
			if(errCount >= 1)
			{
				char szMessage[MAX_MESSAGE + 1] = "";
				rc = cwbSV_GetErrText(errHandle, szMessage, MAX_MESSAGE + 1, NULL);
				if(rc == CWB_OK)
				{
					cout << szMessage << endl << endl;
				}
			}
		}
	}

	//Shut down the remote command job.
	rc = cwbRC_StopSys(rcSysHandle);
    if(rc != CWB_OK)
	{
		cout << "Error ending remote command job!" << endl;
	}

	//Get rid of error handle
	rc = cwbSV_DeleteErrHandle(errHandle);
	if(rc != CWB_OK)
	{
		cout << "Error releasing error handle!" << endl;
	}
}

/****************************************************************/
/*  Function listShares()                                       */
/*  Establishes a connection, creates a user space, dumps       */
/*  Netserver information to the user space, retrieves, and     */
/*  cleans up.                                                  */
/****************************************************************/
void listShares()
{
	unsigned int rc = 0;
	unsigned long errCount = 0;
	cwbSV_ErrHandle errHandle = 0;
	cwbRC_SysHandle rcSysHandle = 0;
    cwbRC_PgmHandle pgmHandle = 0;

	//Create an error handle
	rc = cwbSV_CreateErrHandle(&errHandle);
    if(rc != CWB_OK)
	{
		cout << "Error creating error handle!" << endl;
		return;
	}

	//Start a remote command conversation
	rc = cwbRC_StartSysEx(sysHandle, &rcSysHandle);
	if(rc != CWB_OK)
	{
		cout << "Error starting Remote Command job!" << endl;
		return;
	}

	//Create program to create a user space
	char szProgram[11] = "QUSCRTUS";
	char szLibrary[11] = "QSYS";
	rc = cwbRC_CreatePgm(szProgram, szLibrary, &pgmHandle);
	if(rc != CWB_OK)
	{
		cout << "Error creating program!" << endl;
		return;
	}

	//Add the parameters
	//Parm 1 - User space to create
	unsigned char szName[20] = {0xD5, 0xC5, 0xE3, 0xE2, 0xE5, 0xD9, 0xC5, 0xC4, 0xC9, 0xE3,   //"NETSVREDIT"
		                        0xD8, 0xE3, 0xC5, 0xD4, 0xD7, 0x40, 0x40, 0x40, 0x40, 0x40};  //"QTEMP     "
    
	rc = cwbRC_AddParm(pgmHandle, CWBRC_INPUT, 20, szName);
	if(rc != CWB_OK)
	{
		cout << "Error on QUSCRTUS parameter!" << endl;
		return;
	}

	//Parm 2 - Extended attributes
    unsigned char szAttrib[10] = {0x40, 0x40, 0x40, 0x40, 0x40, 0x40, 0x40, 0x40, 0x40, 0x40}; //"          "
	rc = cwbRC_AddParm(pgmHandle, CWBRC_INPUT, 10, szAttrib);
	if(rc != CWB_OK)
	{
		cout << "Error on QUSCRTUS parameter!" << endl;
		return;
	}

	//Parm 3 - Initial size
	unsigned long lSpaceSize = 0xD0070000;       //Make it 2000 bytes long
	rc = cwbRC_AddParm(pgmHandle, CWBRC_INPUT, 4, (unsigned char*)&lSpaceSize);
	if(rc != CWB_OK)
	{
		cout << "Error on QUSCRTUS parameter!" << endl;
		return;
	}

	//Parm 4 - Initial value of all bytes
	unsigned char cInitial = 0x40;
	rc = cwbRC_AddParm(pgmHandle, CWBRC_INPUT, 1, &cInitial);
	if(rc != CWB_OK)
	{
		cout << "Error on QUSCRTUS parameter!" << endl;
		return;
	}

	//Parm 5 - Initial public authority - *CHANGE
	unsigned char szAuthority[10] = {0x5C, 0xC3, 0xC8, 0xC1, 0xD5, 0xC7, 0xC5, 0x40, 0x40, 0x40};
	rc = cwbRC_AddParm(pgmHandle, CWBRC_INPUT, 10, szAuthority);
	if(rc != CWB_OK)
	{
		cout << "Error on QUSCRTUS parameter!" << endl;
		return;
	}

	//Parm 6 - Description - All blanks
	unsigned char szSpaceDesc[50] = {0x40, 0x40, 0x40, 0x40, 0x40, 0x40, 0x40, 0x40, 0x40, 0x40,
		                             0x40, 0x40, 0x40, 0x40, 0x40, 0x40, 0x40, 0x40, 0x40, 0x40,
								     0x40, 0x40, 0x40, 0x40, 0x40, 0x40, 0x40, 0x40, 0x40, 0x40,
								     0x40, 0x40, 0x40, 0x40, 0x40, 0x40, 0x40, 0x40, 0x40, 0x40,
								     0x40, 0x40, 0x40, 0x40, 0x40, 0x40, 0x40, 0x40, 0x40, 0x40};

	rc = cwbRC_AddParm(pgmHandle, CWBRC_INPUT, 50, szSpaceDesc);
	if(rc != CWB_OK)
	{
		cout << "Error on QUSCRTUS parameter!" << endl;
		return;
	}

	//Execute the program
	rc = cwbRC_CallPgm(rcSysHandle, pgmHandle, errHandle);
    if(rc != CWB_OK)
	{
		cout << "Error creating user space!" << endl;
		
		//Check for any errors
		rc = cwbSV_GetErrCount(errHandle, &errCount);
		if(rc == CWB_OK)
		{
			if(errCount >= 1)
			{
				char szMessage[MAX_MESSAGE + 1] = "";
				rc = cwbSV_GetErrText(errHandle, szMessage, MAX_MESSAGE + 1, NULL);
				if(rc == CWB_OK)
				{
					cout << szMessage << endl << endl;
				}
			}
		}
	}

	//Delete the program
	if ( cwbRC_DeletePgm(pgmHandle) != CWB_OK )
	{
		cout << "Error on cleanup!" << endl;
	}
 
	//Call program to dump Netserver information to user space
	char szCommand[MAX_COMMAND] = "CALL QZLSLSTI PARM('NETSVREDITQTEMP     ' ZLSL0100 *ALL x'00000000')";

	rc = cwbRC_RunCmd(rcSysHandle, szCommand, errHandle);
	if(rc != CWB_OK)
	{
		cout << "Error retrieving information!"  << endl;

		//Check for any errors
		rc = cwbSV_GetErrCount(errHandle, &errCount);
		if(rc == CWB_OK)
		{
			if(errCount >= 1)
			{
				char szMessage[MAX_MESSAGE + 1] = "";
				rc = cwbSV_GetErrText(errHandle, szMessage, MAX_MESSAGE + 1, NULL);
				if(rc == CWB_OK)
				{
					cout << szMessage << endl << endl;
				}
			}
		}
	}

    //Create a program to retrieve the user space information.
	char szProgram2[11] = "QUSRTVUS";
	char szLibrary2[11] = "QSYS";
	rc = cwbRC_CreatePgm(szProgram2, szLibrary2, &pgmHandle);
	if(rc != CWB_OK)
	{
		cout << "Error creating program!" << endl;
		return;
	}

    //Parm 1 - User space name, using szName declared previously
	rc = cwbRC_AddParm(pgmHandle, CWBRC_INPUT, 20, szName);
	if(rc != CWB_OK)
	{
		cout << "Error on QUSRTVUS parameter!" << endl;
		return;
	}

	//Parm 2 - Starting position in user space, first time get offset to data
	unsigned long lStart = 0x7D000000;
	rc = cwbRC_AddParm(pgmHandle, CWBRC_INPUT, 4, (unsigned char*)&lStart);
	if(rc != CWB_OK)
	{
		cout << "Error on QUSRTVUS parameter!" << endl;
		return;
	}

	//Parm 3 - Number of bytes to retrieve
	unsigned long lLength = 0x0C000000;
	rc = cwbRC_AddParm(pgmHandle, CWBRC_INPUT, 4, (unsigned char*)&lLength);
	if(rc != CWB_OK)
	{
		cout << "Error on QUSRTVUS parameter!" << endl;
		return;
	}

	//Parm 4 - Variable to receive information
	unsigned char szOffset[12] = "";
	rc = cwbRC_AddParm(pgmHandle, CWBRC_OUTPUT, 12, szOffset);
	if(rc != CWB_OK)
	{
		cout << "Error on QUSRTVUS parameter!" << endl;
		return;
	}

	//Execute the program
	rc = cwbRC_CallPgm(rcSysHandle, pgmHandle, errHandle);
    if(rc != CWB_OK)
	{
		cout << "Error retrieving user space!" << endl;
		
		//Check for any errors
		rc = cwbSV_GetErrCount(errHandle, &errCount);
		if(rc == CWB_OK)
		{
			if(errCount >= 1)
			{
				char szMessage[MAX_MESSAGE + 1] = "";
				rc = cwbSV_GetErrText(errHandle, szMessage, MAX_MESSAGE + 1, NULL);
				if(rc == CWB_OK)
				{
					cout << szMessage << endl << endl;
				}
			}
		}
	}

	
	//Determine the offset value and number of entries
	unsigned long lOffset = bytesToInt(&szOffset[0]);
	unsigned long lNumberOfShares = bytesToInt(&szOffset[8]);
	
	//Add 1 to the offset since user space index starts at 1
	lOffset = lOffset + 1;

	//Determine code page of process
	unsigned long lPCCodePage = 0;
	rc = cwbNL_GetCodePage(&lPCCodePage, 0);
	if(rc != CWB_OK)
	{
		cout << "Unable to retrieve pc code page!" << endl;
	}

	//Determine code page of AS400
	unsigned long lCCSID = 0;
	rc = cwbNL_GetHostCCSID(szSystem, &lCCSID);
	if(rc != CWB_OK)
	{
		cout << "Unable to retrieve host CCSID!" << endl;
	}

	unsigned long lHostCodePage = 0;
	rc = cwbNL_CCSIDToCodePage(lCCSID, &lHostCodePage, 0);
	if(rc != CWB_OK)
	{
		cout << "Unable to retrieve host code page!" << endl;
	}

    //Create a converter
	cwbNL_Converter cvtr = 0;
	char pad = ' ';
	rc = cwbNL_CreateConverter(lHostCodePage, lPCCodePage, &cvtr, 0, 0, 0, &pad);
	if(rc != CWB_OK)
	{
		cout << "Unable to create a converter!" << endl;
		return;
	}

	unsigned long lShare = 1;
	while(lShare <= lNumberOfShares)
	{
      //Convert to AS400 parameter
	  unsigned long lOffsetParm = 0;
	  rc = cwbDT_Bin4ToBin4((char*)(void*)&lOffsetParm, (char*)(void*)&lOffset);
	  if(rc != CWB_OK)
	  {
		  cout << "Error converting parameter!" << endl;
		  return;
	  }

	  //Change parameters to retrieve at offset.
	  rc = cwbRC_SetParm(pgmHandle, 1, CWBRC_INPUT, 4, (unsigned char*)&lOffsetParm);
	  if(rc != CWB_OK)
	  {
		  cout << "Error assigning parameter!" << endl;
		  return;
	  }

	  unsigned long lDataSize = 0x00020000;
	  rc = cwbRC_SetParm(pgmHandle, 2, CWBRC_INPUT, 4, (unsigned char*)&lDataSize);
	  if(rc != CWB_OK)
	  {
		  cout << "Error assigning parameter!" << endl;
		  return;
	  }

	  unsigned char szInformation[500] = "";
	  rc = cwbRC_SetParm(pgmHandle, 3, CWBRC_OUTPUT, 500, szInformation);
	  if(rc != CWB_OK)
	  {
		cout << "Error assigning parameter!" << endl;
		return;
	  }

	  //Execute the program to retrieve Netserver info
	  rc = cwbRC_CallPgm(rcSysHandle, pgmHandle, errHandle);
      if(rc != CWB_OK)
	  {
		  cout << "Error retrieving user space!" << endl;
		
		  //Check for any errors
		  rc = cwbSV_GetErrCount(errHandle, &errCount);
		  if(rc == CWB_OK)
		  {
			  if(errCount >= 1)
			  {
				  char szMessage[MAX_MESSAGE + 1] = "";
				  rc = cwbSV_GetErrText(errHandle, szMessage, MAX_MESSAGE + 1, NULL);
				  if(rc == CWB_OK)
				  {
					  cout << szMessage << endl << endl;
				  }
			  }
		  }
	  }

	  //Retrieve information from raw data in szInformation
	  unsigned long lNeeded = 0;
	  unsigned long lNumberErrors = 0;
	  unsigned long lErrorIndex = 0;

      //Convert the data
	  unsigned long lEntrySize;
	  unsigned long lShareType;
	  unsigned long lPathLength;
	  unsigned long lPermissions;
	  char szShareName[MAX_SHARE + 1] = "";
	  char szPath[MAX_PATHNAME + 1] = "";

	  lEntrySize = bytesToInt(&szInformation[0]);
	  lShareType = bytesToInt(&szInformation[16]);
	  lPathLength = bytesToInt(&szInformation[40]);
	  lPermissions = bytesToInt(&szInformation[20]);
	  cwbNL_Convert(cvtr, 12, 13, (char*)&szInformation[4], szShareName, &lNumberErrors, &lErrorIndex, &lNeeded, 0);
      cwbNL_Convert(cvtr, lPathLength, MAX_PATHNAME + 1, (char*)&szInformation[164], szPath, &lNumberErrors, &lErrorIndex, &lNeeded, 0);

	  if(lShareType==0) //File Share?
	  {
	    cout << "Share Name:     " << szShareName << endl;
	    cout << "Path Name:      " << szPath << endl;
		cout << "Permissions:    ";
		if(lPermissions==1)
		{
			cout << "Read Only" << endl;
		}
		else
		{
			cout << "Read/Write" << endl;
		}
		cout << endl;
	  }

	  //Increment share number
	  lShare++;
	  
	  //Increment offset by EntrySize
	  lOffset = lOffset + lEntrySize;
    }

	//Cleanup
	//Delete the converter
	rc = cwbNL_DeleteConverter(cvtr, 0);
	if(rc != CWB_OK)
	{
		cout << "Error deleting converter!" << endl;
	}

    //Delete the user space.
	char szCommand2[MAX_COMMAND] = "DLTUSRSPC QTEMP/NETSVREDIT";

	rc = cwbRC_RunCmd(rcSysHandle, szCommand2, errHandle);
	if(rc != CWB_OK)
	{
		cout << "Error on cleanup!"  << endl;

		//Check for any errors
		rc = cwbSV_GetErrCount(errHandle, &errCount);
		if(rc == CWB_OK)
		{
			if(errCount >= 1)
			{
				char szMessage[MAX_MESSAGE + 1] = "";
				rc = cwbSV_GetErrText(errHandle, szMessage, MAX_MESSAGE + 1, NULL);
				if(rc == CWB_OK)
				{
					cout << szMessage << endl << endl;
				}
			}
		}
	}

	//Shut down the remote command job.
	rc = cwbRC_StopSys(rcSysHandle);
    if(rc != CWB_OK)
	{
		cout << "Error ending remote command job!" << endl;
	}

	//Get rid of error handle
	rc = cwbSV_DeleteErrHandle(errHandle);
	if(rc != CWB_OK)
	{
		cout << "Error releasing error handle!" << endl;
	}
}

void intToHex(int i)
{
	szBuffer[0] = '0';
	szBuffer[1] = '0';
	szBuffer[2] = '0';
	szBuffer[3] = '0';
	szBuffer[4] = '0';

    int iDivisor = 0;
	char cDigit = '0';
    
	if (i > 256)
	{
		iDivisor = i/256;
		cDigit = getDigit(iDivisor);
		szBuffer[5] = cDigit;
	}
	else
	{
		szBuffer[5] = '0';
	}
	i = i - (256*iDivisor);

	if (i > 16)
	{
		iDivisor = i/16;
		cDigit = getDigit(iDivisor);
		szBuffer[6] = cDigit;
	}
	else
	{
		szBuffer[6] = '0';
	}
    i = i - (16*iDivisor);

    cDigit = getDigit(i);
	szBuffer[7] = cDigit;
	szBuffer[8] = '\0';
}

char getDigit(int i)
{
	char result = ' ';

	switch(i)
	{
	case 0:
		result = '0';
		break;
	case 1:
		result = '1';
		break;
	case 2:
		result = '2';
		break;
	case 3:
		result = '3';
		break;
	case 4:
		result = '4';
		break;
	case 5:
		result = '5';
		break;
	case 6:
		result = '6';
		break;
	case 7:
		result = '7';
		break;
	case 8:
		result = '8';
		break;
	case 9:
		result = '9';
		break;
	case 10:
		result = 'A';
		break;
	case 11:
		result = 'B';
		break;
	case 12:
		result = 'C';
		break;
	case 13:
		result = 'D';
		break;
	case 14:
		result = 'E';
		break;
	case 15:
		result = 'F';
		break;
	default:
		result = '0';
	}

	return result;
}

unsigned long bytesToInt(unsigned char* byteArray)
{
	unsigned long result = 0;
	result = *(byteArray)*(16777216);           //Get first byte
	result = result + *(byteArray+1)*(65536);   //Get second byte
	result = result + *(byteArray+2)*(256);     //Get third byte
	result = result + *(byteArray+3);           //Get fourth and final byte
	return result;
}